﻿// ReSharper disable MemberCanBePrivate.Global
// ReSharper disable UnusedAutoPropertyAccessor.Global
using anbs_cp.Enums;
using anbs_cp.Interfaces;

namespace anbs_cp.Classes;

/*
    История версий

    Обновлено 2024.03.26
    * Все интерфейсы объеденены в один IActionStateMessage
    
    Обновлено 2023.12.10
    * Заменены классы Action* на соответствующие интерфейсы
    * Класс ActionState теперь наследует интерфейс IActionState, позволяющий добавлять состояния (ошибки, предупреждения и информацию)
        из разных классов ActionState, т.е. например, данные из ActionState<bool> (кроме значения!) можно перенести в ActionState<int>.

    Обновлено 2023.11.1
    * Добавлен возвращаемый результат

    Обновлено 2023.01.121.1:
    * Заменены интерфейсы IAction* на соответствующие классы
*/

/// <summary>
/// Класс, хранящий в себе состояния действия
/// </summary>
/// <typeparam name="T">Тип значения - результата действия</typeparam>
public class ActionState<T>: IActionState
{
    /// <summary>
    /// Конструктор по умолчанию
    /// </summary>
    // ReSharper disable once MemberCanBeProtected.Global
    public ActionState ()
    {
        Info = [];
        Warnings = [];
        Errors = [];
        Value = default;
    }

    /// <summary>
    /// Конструктор с указанием начального значения <see cref="Value"/>
    /// </summary>
    /// <param name="defaultValue">Начальное значение <see cref="Value"/></param>
    public ActionState (T defaultValue)
    {
        Info = [];
        Warnings = [];
        Errors = [];
        Value = defaultValue;
    }

    /// <summary>
    /// Список информации
    /// </summary>
    public List<IActionStateMessage> Info { get; }

    /// <summary>
    /// Список предупреждений
    /// </summary>
    public List<IActionStateMessage> Warnings { get; }

    /// <summary>
    /// Список ошибок
    /// </summary>
    public List<IActionStateMessage> Errors { get; }

    /// <summary>
    /// Значение
    /// </summary>
    public T? Value { get; set; }

    #region Методы

    #region Очистка
    /// <summary>
    /// Очищает список ошибок
    /// </summary>
    public void ClearErrors ()
    {
        Errors.Clear();
        Errors.TrimExcess();
    }

    /// <summary>
    /// Очищает список предупреждений
    /// </summary>
    public void ClearWarnings ()
    {
        Warnings.Clear();
        Warnings.TrimExcess();
    }

    /// <summary>
    /// Очищает список информации
    /// </summary>
    public void ClearInfo ()
    {
        Info.Clear();
        Info.TrimExcess();
    }

    /// <summary>
    /// Очищает все списки
    /// </summary>
    public void Clear ()
    {
        ClearInfo();
        ClearWarnings();
        ClearErrors();
    }
    #endregion

    #region Добавление ошибки
    /// <summary>
    /// Добавление ошибки
    /// </summary>
    /// <param name="error">Ошибка</param>
    // ReSharper disable once MemberCanBeMadeStatic.Global
    // ReSharper disable once FunctionRecursiveOnAllPaths
    public void AddError (IActionStateMessage error) => Errors.Add(error);

    /// <summary>
    /// Добавляет ошибки в список
    /// </summary>
    /// <param name="errors">Список ошибок</param>
    public void AddErrors (IEnumerable<IActionStateMessage> errors) => Errors.AddRange(errors);

    /// <summary>
    /// Добавление ошибки
    /// </summary>
    /// <param name="critical">Является ли ошибка критической</param>
    public void AddError (bool critical = true)
    {
        //Создаю ошибку
        ActionStateMessage error = new("", critical);

        //Добавляю ошибку
        AddError(error);
    }

    /// <summary>
    /// Добавление ошибки
    /// </summary>
    /// <param name="message">Сообщение об ошибке</param>
    /// <param name="critical">Является ли ошибка критической</param>
    public void AddError (string message, bool critical = true)
    {
        //Создаю ошибку
        ActionStateMessage error = new(message, critical);

        //Добавляю ошибку
        AddError(error);
    }

    /// <summary>
    /// Добавление ошибки
    /// </summary>
    /// <param name="errorObject">Объект ошибки</param>
    /// <param name="message">Сообщение об ошибке</param>
    /// /// <param name="critical">Является ли ошибка критической</param>
    public void AddError (string errorObject, string message, bool critical = true)
    {
        //Создаю ошибку
        ActionStateMessage error = new(errorObject, message, critical);

        //Добавляю ошибку
        AddError(error);
    }
    #endregion

    #region Добавление предупреждения
    /// <summary>
    /// Добавление предупреждения
    /// </summary>
    /// <param name="warning">Предупреждение</param>
    public void AddWarning (IActionStateMessage warning) => Warnings.Add(warning);

    /// <summary>
    /// Добавление предупреждений
    /// </summary>
    /// <param name="warnings">Список предупреждений</param>
    public void AddWarnings (IEnumerable<IActionStateMessage> warnings) => Warnings.AddRange(warnings);

    /// <summary>
    /// Добавление предупреждение
    /// </summary>
    /// <param name="message">Текст предупреждения</param>
    /// <param name="warningObject">Объект предупреждения</param>
    public void AddWarning (string message, string warningObject = "")
    {
        //Создаю предупреждение
        ActionStateMessage warning = new(warningObject, message);

        //Добавляю предупреждение
        AddWarning(warning);
    }
    #endregion

    #region Добавление информации
    /// <summary>
    /// Добавление информации
    /// </summary>
    /// <param name="info">Информация</param>
    public void AddInfo (IActionStateMessage info) => Info.Add(info);

    /// <summary>
    /// Добавление информации
    /// </summary>
    /// <param name="infos">Список информации</param>
    public void AddInfos (IEnumerable<IActionStateMessage> infos) => Info.AddRange(infos);

    /// <summary>
    /// Добавление информации
    /// </summary>
    /// <param name="message">Текст информации</param>
    /// <param name="infoObject">Объект информации</param>
    public void AddInfo (string message, string infoObject = "")
    {
        //Создаю информацию
        ActionStateMessage info = new(infoObject, message);

        //Добавляю информацию
        AddInfo(info);
    }
    #endregion

    #region Печать
    /// <summary>
    /// Печать сообщений
    /// </summary>
    /// <param name="area">Область печати</param>
    /// <param name="formatList">Оформление всей области</param>
    /// <param name="formatItem">Оформление элемента</param>
    /// <returns>Текстовое представление выбранных списокв</returns>
    /// <exception cref="ArgumentOutOfRangeException">Ошибка при неизвестной области</exception>
    public string PrintMessage (EActionStatePrintArea area, string formatList, string formatItem)
    {
        // Создаю список
        List<IActionStateMessage> list = [];

        // Выбираю списки по областям
        switch (area)
        {
            case EActionStatePrintArea.All:
                list.AddRange(Errors);
                list.AddRange(Warnings);
                list.AddRange(Info);
                break;
            case EActionStatePrintArea.ErrorsAndWarnings:
                list.AddRange(Errors);
                list.AddRange(Warnings);
                break;
            case EActionStatePrintArea.ErrorsOnly:
                list.AddRange(Errors);
                break;
            case EActionStatePrintArea.WarningsOnly:
                list.AddRange(Warnings);
                break;
            case EActionStatePrintArea.InfosOnly:
                list.AddRange(Info);
                break;
            default:
                throw new ArgumentOutOfRangeException(nameof(area), area, null);
        }

        // Создаю элементы
        string elements =
            list.Aggregate(string.Empty, (current, item) => current + item.PrintMessage(formatItem));

        return string.Format(formatList, elements);
    }
    #endregion

    #region Проверка на наличие
    /// <summary>
    /// Проверяет, есть ли ошибки
    /// </summary>
    /// <param name="ignoreNonCritical">Игнорировать не критические</param>
    /// <returns>Наличие ошибок</returns>
    public bool HasErrors (bool ignoreNonCritical = false) =>
        ignoreNonCritical ? Errors.Any(static error => error.IsCritical) : Errors.Any();

    /// <summary>
    /// Проверяет, есть ли предупреждения
    /// </summary>
    /// <param name="ignoreInformWarning">Игнорировать информационные предупреждения</param>
    /// <returns>Наличие предупреждений</returns>
    public bool HasWarnings (bool ignoreInformWarning = false) => ignoreInformWarning
        ? Warnings.Any(static warning => warning.IsCritical)
        : Warnings.Any();

    /// <summary>
    /// Проверяет, есть ли сообщения
    /// </summary>
    /// <param name="ignoreStatus">Игнорировать статусные сообщения</param>
    /// <returns>Наличие сообщений</returns>
    public bool HasInfo (bool ignoreStatus) => ignoreStatus ? Info.Any(static info => info.IsCritical) : Info.Any();

    /// <summary>
    /// Успешно ли завершилось
    /// </summary>
    public bool IsSuccess (bool ignoreNonCriticalErrors = false) =>
        !HasErrors(ignoreNonCriticalErrors) && !HasWarnings(true);
    #endregion

    #region Количество сообщений
    /// <summary>
    /// Количество ошибок
    /// </summary>
    /// <param name="ignoreNonCritical">Игнорировать не критические</param>
    /// <returns>Количество ошибок</returns>
    public int ErrorsCount (bool ignoreNonCritical = false) =>
        ignoreNonCritical ? Errors.Count(static error => error.IsCritical) : Errors.Count;

    /// <summary>
    /// Количество предупреждений
    /// </summary>
    /// <param name="ignoreInformWarning">Игнорировать информационные предупреждения</param>
    /// <returns>Количество предупреждений</returns>
    public int WarningsCount (bool ignoreInformWarning = false) => ignoreInformWarning
        ? Warnings.Count(static warning => warning.IsCritical)
        : Warnings.Count;

    /// <summary>
    /// Количество информационных сообщений
    /// </summary>
    /// <param name="ignoreStatus">Игнорировать статусные сообщения</param>
    /// <returns>Количество информационных сообщений</returns>
    public int InfoCount (bool ignoreStatus) => ignoreStatus
        ? Info.Count(static info => info.IsCritical)
        : Info.Count;
    #endregion

    #region Добавление другого состояния

    /// <summary>
    /// Добавляет другое состояние (например, результат другого действия, который возвращает <see cref="ActionState"/>).
    /// </summary>
    /// <typeparam name="T">Тип параметра</typeparam>
    /// <param name="state">Запись состояния</param>
    public void AddState (IActionState state)
    {
        AddErrors(state.Errors);
        AddWarnings(state.Warnings);
        AddInfos(state.Info);
    }
    #endregion
    #endregion
}